#!/usr/bin/perl -w

# tsv2db: converts delimiter-separated tables into DocBook CALS tables
# Purpose: to import tables generated by spreadsheets like OO calc
# into DocBook documents.

# usage: tsv2db < infile.tsv > outfile.xml

# -d dbversion: specify the DocBook version (default: 5.0)
# -e encoding: specify xml encoding (default: utf-8)
# -f source: apply source-dependent preprocessing (oocalc|excel)
# -h display help and exit
# -H if set, input data do NOT have a header
# -l lang: set language (default: en)
# -n namespace: set namespace string (default http://docbook.org/ns/docbook)
# -p prefix: set namespace prefix (default: none)
# -s separator: override default input column separator (tab)
# -t title: specify the table title
# -x id : set xml:id

# external dependencies: Getopt::Std

# limitations: works best with tab-separated columns. May create
# undesired results if the separator is also a decimal separator, if
# the fields use escaped text delimiters, or if any other advanced
# feature of the csv format are used. All of these should not be a
# problem for your run-of-the-mill Calc or Excel-exported table

# todo: should be rewritten using Text::CSV, but I loathe the
# overhead. Also, reading csv data from stdin is not possible with
# that module


# use this module to read command line options
use Getopt::Std;

# variables that hold the command line options
my $dbversion = "5.0";
my $dbencoding = "utf-8";
my $preprocessing = "";
my $header = 1;
my $dblang = "en";
my $dbns = "http://docbook.org/ns/docbook";
my $dbprefix = "";
my $separator = "\t";
my $dbtitle = "";
my $dbxmlid = "";

# derived and other variables
my $dbdeclprefix = "";
my $have_header;
my @entries;
my $maxnum_entries = 0;
my $outstring;

# this hash will receive the command line options
my %opts;

getopts('d:e:f:hl:np:s:t:x:', \%opts);

## loop over all command line options
while (($key, $value) = each %opts) {
    if ($key eq "d") {
	$dbversion = $value;
    }
    elsif ($key eq "e") {
	$dbencoding = $value;
    }
    elsif ($key eq "f") {
	$preprocessing = $value;
	if ($preprocessing ne "oocalc") {
	    print(STDERR "no preprocessing available for $preprocessing\n");
	}
    }
    elsif ($key eq "h") {
	print("tsv2db converts tab-separated tables into DocBook CALS tables
Usage:   tsv2db < infile.tsv > outfile.xml
Options: -d dbversion: specify the DocBook version
         -f source: apply source-dependent preprocessing (oocalc|excel)
         -h display help and exit
         -H if set, input data do NOT have a header
         -l lang: set language
         -n namespace: set namespace string (default: http://docbook.org/ns/docbook)
         -p prefix: set namespace prefix
         -s separator: override default input column separator (tab)
         -t title: specify the table title
         -x id : set xml:id\n");
	exit (0);
    }
    elsif ($key eq "H") {
	$header = 0;
    }
    elsif ($key eq "l") {
	$dblang = $value;
    }
    elsif ($key eq "n") {
	$dbns = $value;
    }
    elsif ($key eq "p") {
	$dbprefix = $value . ":";
	$dbdeclprefix = ":" . $value;
    }
    elsif ($key eq "s") {
	$separator = $value;
	if ($separator eq "tab") {
	    $separator = "\t";
	}
    }
    elsif ($key eq "t") {
	$dbtitle = $value;
    }
    elsif ($key eq "x") {
	$dbxmlid = $value;
    }
    # else: getopt takes care of the rest
}


# loop over input data
while (<>) {
    $_ =~ s/\r|\n//g; # like chomp, but gets rid of cr as well (hi windoze!)

    @entries = split(/$separator/);
    $maxnum_entries = (@entries > $maxnum_entries) ? @entries : $maxnum_entries;

    if ($header == 1) {
	$outstring .= "    <${dbprefix}thead>\n";
	$have_header++;
    }

    $outstring .= "      <${dbprefix}row>\n";

    foreach my $entry (@entries) {
	if ($preprocessing) {
	    # currently only OpenOffice/LibreOffice calc is supported
	    if ($preprocessing eq "oocalc") {
		# OO exports cells as text enclosed in ""
		# with trailing spaces
		$entry =~ s/^\"(.*)\"$/$1/;
		$entry =~ s/([^\s]*)\s+$/$1/;
	    }
	}

	$outstring .= "        <${dbprefix}entry>" . $entry . "</${dbprefix}entry>\n";
    }

    $outstring .= "      </${dbprefix}row>\n";

    if ($header == 1) {
	$outstring .= "    </${dbprefix}thead>\n    <${dbprefix}tbody>\n";
	$header = 0;
    }
}

# finish table
$outstring .= "    </${dbprefix}tbody>\n  </${dbprefix}tgroup>\n</${dbprefix}table>\n";

# print table header
print("<?xml version=\"1.0\" encoding=\"$dbencoding\"?>
<${dbprefix}table xmlns${dbdeclprefix}=\"$dbns\"
       version=\"$dbversion\"
       xml:id=\"$dbxmlid\"
       xml:lang=\"$dblang\"
       colsep=\"0\"
       rowsep=\"0\">
  <${dbprefix}title>$dbtitle</${dbprefix}title>
  <${dbprefix}tgroup cols=\"$maxnum_entries\">\n");

if ($have_header == 0) {
    print("    <${dbprefix}tbody>\n");
}

print($outstring);

exit (0);
